local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( "Broker_Money")
local LibQTip = LibStub('LibQTip-1.0')
local L_GOLD = L["g"]
local L_SILVER = L["s"]
local L_COPPER = L["c"]


local Money = CreateFrame("frame")
LibStub("AceEvent-3.0"):Embed(Money)

Money.dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(L["Broker_Money"], 
  { type = "data source", 
   label = L["Broker Money"], 
	icon = "Interface\\MoneyFrame\\UI-GoldIcon",
	text = "5",
   })
   
local options = {
	name = L["Broker Money"],
	type = "group",
	args = {
		confdesc = {
			order = 1,
			type = "description",
			name = L["Simple money display for LDB\n"],
			cmdHidden = true
		},
		showCoinIcons = {
			order = 2, 
			type = "toggle", 
			width = "full",
			name = L["Show coin icons"],
			desc = L["Shows coin icons"],
			get = function() return Broker_MoneyConfig.ShowCoinIcons end,
			set = function(_,v) Broker_MoneyConfig.ShowCoinIcons = v 
				Money:UpdateConfig()
			end,
		},
		iconSize = {
			type = "range",
			order = 3,
			name = L["Icon size"],
			desc = L["Change Icon size"],
			min = 1, max = 32, step = 1, bigStep = 1,
			set = function(_,v) Broker_MoneyConfig.IconSize = v 
				Money:UpdateConfig()
			end,
			get = function() return Broker_MoneyConfig.IconSize end,
			disabled = function() return not Broker_MoneyConfig.ShowCoinIcons end,
		},
		showColoredText = {
			order = 4, 
			type = "toggle", 
			width = "full",
			name = L["Show colored text"],
			desc = L["Shows colored text"],
			get = function() return Broker_MoneyConfig.ShowColoredText end,
			set = function(_,v) Broker_MoneyConfig.ShowColoredText = v 
				Money:UpdateConfig()
			end,
		},				
		compressDisplay = {
			order = 5, 
			type = "toggle", 
			width = "full",
			name = L["Compress display"],
			desc = L["Display only relevant information (Value not 0)"],
			get = function() return Broker_MoneyConfig.CompressDisplay end,
			set = function(_,v) Broker_MoneyConfig.CompressDisplay = v 
				Money:UpdateConfig()
			end,
		},			
	}
}	
   
LibStub("AceConfig-3.0"):RegisterOptionsTable(L["Broker Money"], options)
LibStub("AceConfigDialog-3.0"):AddToBlizOptions(L["Broker Money"])

local currentMoney, toolTipRef
local goldicon, silvericon, coppericon
local COLOR_COPPER, COLOR_SILVER, COLOR_GOLD
local gold, silver, copper

local playerName = UnitName("player")
local realmName = GetRealmName()

function Money:UpdateConfig() 
	if not Broker_MoneyConfig then 
		Broker_MoneyConfig = { 		
			ShowCoinIcons = true,
			ShowColoredText = true,
			CompressDisplay = false,
			IconSize = 16,
        }
  	end  	
	
	if not Broker_MoneyDB or not Broker_MoneyDB.realm then
		Broker_MoneyDB = {			
			realm = {},
		}
	end
	if not Broker_MoneyDB.realm[realmName] then
		Broker_MoneyDB.realm[realmName] = {}
	end
		
	if Broker_MoneyConfig.ShowCoinIcons then
		--use built-in icons
		--goldicon = " |TInterface\\AddOns\\Broker_Money\\Icons\\UI-GoldIcon::|t"
		--silvericon = " |TInterface\\AddOns\\Broker_Money\\Icons\\UI-SilverIcon::|t"
		--coppericon = " |TInterface\\AddOns\\Broker_Money\\Icons\\UI-CopperIcon::|t"
		
		-- 16 is fine
		local iconSize
		if not Broker_MoneyConfig.IconSize then		
			Broker_MoneyConfig.IconSize = 16
		end
		
		iconSize = Broker_MoneyConfig.IconSize
		
		goldicon = string.format("\124TInterface\\MoneyFrame\\UI-GoldIcon:%d:%d:1:0\124t", iconSize, iconSize)
		silvericon = string.format("\124TInterface\\MoneyFrame\\UI-SilverIcon:%d:%d:1:0\124t", iconSize, iconSize)
		coppericon = string.format("\124TInterface\\MoneyFrame\\UI-CopperIcon:%d:%d:1:0\124t", iconSize, iconSize)
	else
		goldicon = string.format("|cffffd700%s|r", L_GOLD)
		silvericon = string.format("|cffc7c7cf%s|r", L_SILVER)
		coppericon = string.format("|cffeda55f%s|r", L_COPPER)
	end
	if Broker_MoneyConfig.ShowColoredText then
		COLOR_GOLD = "|cffffd700%i|r"
		COLOR_SILVER = "|cffc7c7cf%i|r"
		COLOR_COPPER = "|cffeda55f%i|r"		
	else 
		COLOR_COPPER = "%i"
		COLOR_SILVER = "%i"
		COLOR_GOLD = "%i"
	end
	
	Money:Update()
end

local function GetTipAnchor(frame)
	local x,y = frame:GetCenter()
	if not x or not y then return "TOPLEFT", "BOTTOMLEFT" end
	local hhalf = (x > UIParent:GetWidth()*2/3) and "RIGHT" or (x < UIParent:GetWidth()/3) and "LEFT" or ""
	local vhalf = (y > UIParent:GetHeight()/2) and "TOP" or "BOTTOM"
	return vhalf..hhalf, frame, (vhalf == "TOP" and "BOTTOM" or "TOP")..hhalf
end

function Money.dataobj.OnLeave()
   -- Release the tooltip
   LibQTip:Release(toolTipRef)
   toolTipRef = nil
end

function Money.dataobj.OnEnter(self)
	local tooltip = LibQTip:Acquire("BrokerMoneyTooltip", 4, "LEFT", "RIGHT", "RIGHT", "RIGHT")
	toolTipRef = tooltip 
   
	tooltip:AddHeader('Broker_Money')
	tooltip:AddLine(" ")
	
	local totalMoney = 0
	
	for name,value in pairs(Broker_MoneyDB.realm[realmName]) do		
		local moneyTable = Money:CreateMoneyStrings(value)
		
		tooltip:AddLine(string.format("%s: ", name), 
			moneyTable["gold"], 
			moneyTable["silver"], 
			moneyTable["copper"])
		totalMoney = totalMoney + value
	end

	tooltip:AddLine(" ")

	local moneyTable = Money:CreateMoneyStrings(totalMoney)    
	tooltip:AddLine("Total: ", 
		moneyTable["gold"], 
		moneyTable["silver"], 
		moneyTable["copper"])
	
   tooltip:SmartAnchorTo(self)
   
   tooltip:Show()
end

function Money:CreateMoneyStrings(currentMoney) 
	gold = floor(currentMoney / 10000)
	silver = floor((currentMoney - (gold * 10000)) / 100)
	copper = mod(currentMoney, 100)
	
	local goldString, silverString, copperString
	
	moneystring = ""
	local CompressDisplay = Broker_MoneyConfig.CompressDisplay
	
	if(gold > 0 or not CompressDisplay) then
		goldString = string.format("%s%s ",  string.format(COLOR_GOLD, gold), goldicon)
		moneystring = goldString
	end
	
	if((silver > 0 or gold > 0) or not CompressDisplay) then
		silverString = string.format("%s%s ", string.format(COLOR_SILVER, silver), silvericon)
		moneystring = moneystring..silverString;
	end
	
	copperString = string.format("%s%s", string.format(COLOR_COPPER, copper), coppericon)
	moneystring = moneystring..copperString
	
	moneyTable = { full=moneystring,  gold=goldString, silver=silverString, copper=copperString }
	
	return moneyTable
end

function Money:Update()
	local currentMoney = GetMoney()
	
	Broker_MoneyDB.realm[realmName][playerName] = currentMoney
	local moneyTable = Money:CreateMoneyStrings(currentMoney)
	Money.dataobj.text = moneyTable["full"]
	
	if tipshown then dataobj.OnEnter(tipshown) end
end

Money:RegisterEvent("PLAYER_MONEY", "Update")
Money:RegisterEvent("PLAYER_TRADE_MONEY", "Update")
Money:RegisterEvent("TRADE_MONEY_CHANGED", "Update")
Money:RegisterEvent("SEND_MAIL_MONEY_CHANGED", "Update")
Money:RegisterEvent("SEND_MAIL_COD_CHANGED", "Update")
Money:RegisterEvent("PLAYER_LOGIN", "UpdateConfig")

Money:UpdateConfig()